#ifndef _CREATETRIGGERS_CPP
#define _CREATETRIGGERS_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <WindowsX.H>
#include <ShellAPI.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <SQL.H>
#include <SQLExt.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../Resources/Resource.H"
#include "../../SharedClasses/CRC32/CRC.H"
#include "../../SharedClasses/SQLClass/cSQL.H"
#include "../../SharedClasses/SQLClass/cRecordSet.H"
#include "../CSockSrvr/CSockSrvr.H"

#include "NSWFL.H"
#include "Init.H"
#include "Entry.H"
#include "Routines.H"
#include "Console.H"
#include "CreateTriggers.H"

#include "../Dialogs/ReplicationDlg.H"
#include "../Dialogs/MainDlg.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool DropTriggers(CSQL *lpCSQL, char *sTable)
{
	char sSQL[1024];

	sprintf(sSQL, "DROP TRIGGER [SQLExch_%s_INS_Trigger]", sTable);
	lpCSQL->ExecuteNonQuery(sSQL);

	sprintf(sSQL, "DROP TRIGGER [SQLExch_%s_UPD_Trigger]", sTable);
	lpCSQL->ExecuteNonQuery(sSQL);

	sprintf(sSQL, "DROP TRIGGER [SQLExch_%s_DEL_Trigger]", sTable);
	lpCSQL->ExecuteNonQuery(sSQL);

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool DropTables(CSQL *lpCSQL, char *sDB, char *sDBO, char *sTable)
{
	char sSQL[1024];

	sprintf(sSQL, "DROP TABLE %s.%s.SQLExch_%s_INS", sDB, sDBO, sTable);
	lpCSQL->ExecuteNonQuery(sSQL);

	sprintf(sSQL, "DROP TABLE %s.%s.SQLExch_%s_UPD", sDB, sDBO, sTable);
	lpCSQL->ExecuteNonQuery(sSQL);

	sprintf(sSQL, "DROP TABLE %s.%s.SQLExch_%s_DEL", sDB, sDBO, sTable);
	lpCSQL->ExecuteNonQuery(sSQL);

	return true;
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CreateTables(CSQL *lpCSQL, char *sDB, char *sDBO, char *sTable, char **sPKs, int iPKs, char *sPKColList)
{
	int iRows = 0;
	int iLen = 0;
	char sSQL[1024];
	char sColumn[255];
	char sType[255];

	CRecordSet rsID;

	//-------------------------------------------------------------
	//Create the tables.
	//-------------------------------------------------------------

	sprintf(sSQL, "SELECT * INTO %s.%s.SQLExch_%s_INS FROM %s.%s.%s WHERE 0 = 1",
		sDB, sDBO, sTable, sDB, sDBO, sTable);
	lpCSQL->ExecuteNonQuery(sSQL);

	sprintf(sSQL, "SELECT * INTO %s.%s.SQLExch_%s_UPD FROM %s.%s.%s WHERE 0 = 1",
		sDB, sDBO, sTable, sDB, sDBO, sTable);
	lpCSQL->ExecuteNonQuery(sSQL);

	sprintf(sSQL, "SELECT %s INTO %s.%s.SQLExch_%s_DEL FROM %s.%s.%s WHERE 0 = 1",
		sPKColList, sDB, sDBO, sTable, sDB, sDBO, sTable);
	lpCSQL->ExecuteNonQuery(sSQL);

	//-------------------------------------------------------------
	//Need to add a SQLExch_Pending Bit column to each of the new tables.
	//	We use this column because of the following scenario:
	//		
	//		1) When a transfer begins we need to update SQLExch_Pending = 1.
	//		2) Next we need to select all of the rows where SQLExch_Pending = 1.
	//		3) While the Server and client excange data, and do imports and exports,
	//			the data may have changed, adding more rows to the table with a
	//			SQLExch_Pending flag equal to 0
	//		4) If the impot was a success then we delete from the table where SQLExch_Pending = 1
	//		5) The rows that were added durring the server/client conversation are
	//			preserved for the next transfer
	//-------------------------------------------------------------
	sprintf(sSQL, "ALTER TABLE SQLExch_%s_INS\r\n"
		"ADD [SQLExch_Pending] Bit", sTable);
	lpCSQL->ExecuteNonQuery(sSQL);

	sprintf(sSQL, "ALTER TABLE SQLExch_%s_UPD\r\n"
		"ADD [SQLExch_Pending] Bit", sTable);
	lpCSQL->ExecuteNonQuery(sSQL);

	sprintf(sSQL, "ALTER TABLE SQLExch_%s_DEL\r\n"
		"ADD [SQLExch_Pending] Bit", sTable);
	lpCSQL->ExecuteNonQuery(sSQL);

	//-------------------------------------------------------------
	//Get a list of all of the identity columns in the table.
	sprintf(sSQL, "SELECT SysColumns.Name, SysTypes.Name"
		" FROM SysColumns, SysObjects, SysTypes"
		" WHERE SysObjects.ID = SysColumns.ID AND"
		" SysTypes.xType = SysColumns.xType"
		" AND SysObjects.Name = 'SQLExch_%s_INS'"
		" AND ColStat = 1", sTable);
	lpCSQL->Execute(sSQL, &rsID);
	
	iRows = rsID.RowCount;

	//Drop and recreate all of the ID columns.
	while(rsID.Fetch())
	{
		rsID.sColumnEx(1, sColumn, sizeof(sColumn), &iLen);
		rsID.sColumnEx(2, sType, sizeof(sType), &iLen);

		if(iRows == 1)
		{
			//This is dirty, but we need to add a temp column just
			//	incase the table only contains one column.
			sprintf(sSQL, "ALTER TABLE SQLExch_%s_INS\r\n"
				"ADD [%s_TEMP] %s", sTable, sColumn, sType);
			lpCSQL->ExecuteNonQuery(sSQL);
		}
	
		sprintf(sSQL, "ALTER TABLE SQLExch_%s_INS\r\n"
			"DROP COLUMN [%s]", sTable, sColumn);
		lpCSQL->ExecuteNonQuery(sSQL);

		sprintf(sSQL, "ALTER TABLE SQLExch_%s_INS\r\n"
			"ADD [%s] %s", sTable, sColumn, sType);
		lpCSQL->ExecuteNonQuery(sSQL);

		if(iRows == 1)
		{
			//Drop the temp column.
			sprintf(sSQL, "ALTER TABLE SQLExch_%s_INS\r\n"
				"DROP COLUMN [%s_TEMP]", sTable, sColumn);
			lpCSQL->ExecuteNonQuery(sSQL);
		}
	}
	rsID.Close();

	//-------------------------------------------------------------
	//Get a list of all of the identity columns in the table.
	sprintf(sSQL, "SELECT SysColumns.Name, SysTypes.Name"
		" FROM SysColumns, SysObjects, SysTypes"
		" WHERE SysObjects.ID = SysColumns.ID AND"
		" SysTypes.xType = SysColumns.xType"
		" AND SysObjects.Name = 'SQLExch_%s_UPD'"
		" AND ColStat = 1", sTable);
	lpCSQL->Execute(sSQL, &rsID);

	iRows = rsID.RowCount;

	//Drop and recreate all of the ID columns.
	while(rsID.Fetch())
	{
		rsID.sColumnEx(1, sColumn, sizeof(sColumn), &iLen);
		rsID.sColumnEx(2, sType, sizeof(sType), &iLen);

		if(iRows == 1)
		{
			//This is dirty, but we need to add a temp column just
			//	incase the table only contains one column.
			sprintf(sSQL, "ALTER TABLE SQLExch_%s_UPD\r\n"
				"ADD [%s_TEMP] %s", sTable, sColumn, sType);
			lpCSQL->ExecuteNonQuery(sSQL);
		}

		sprintf(sSQL, "ALTER TABLE SQLExch_%s_UPD\r\n"
			"DROP COLUMN [%s]", sTable, sColumn);
		lpCSQL->ExecuteNonQuery(sSQL);

		sprintf(sSQL, "ALTER TABLE SQLExch_%s_UPD\r\n"
			"ADD [%s] %s", sTable, sColumn, sType);
		lpCSQL->ExecuteNonQuery(sSQL);

		if(iRows == 1)
		{
			//Drop the temp column.
			sprintf(sSQL, "ALTER TABLE SQLExch_%s_UPD\r\n"
				"DROP COLUMN [%s_TEMP]", sTable, sColumn);
			lpCSQL->ExecuteNonQuery(sSQL);
		}
	}
	rsID.Close();

	//-------------------------------------------------------------
	//Get a list of all of the identity columns in the table.
	sprintf(sSQL, "SELECT SysColumns.Name, SysTypes.Name"
		" FROM SysColumns, SysObjects, SysTypes"
		" WHERE SysObjects.ID = SysColumns.ID AND"
		" SysTypes.xType = SysColumns.xType"
		" AND SysObjects.Name = 'SQLExch_%s_DEL'"
		" AND ColStat = 1", sTable);
	lpCSQL->Execute(sSQL, &rsID);

	iRows = rsID.RowCount;

	//Drop and recreate all of the ID columns.
	while(rsID.Fetch())
	{
		rsID.sColumnEx(1, sColumn, sizeof(sColumn), &iLen);
		rsID.sColumnEx(2, sType, sizeof(sType), &iLen);

		if(iRows == 1)
		{
			//This is dirty, but we need to add a temp column just
			//	incase the table only contains one column.
			sprintf(sSQL, "ALTER TABLE SQLExch_%s_DEL\r\n"
				"ADD [%s_TEMP] %s", sTable, sColumn, sType);
			lpCSQL->ExecuteNonQuery(sSQL);
		}

		sprintf(sSQL, "ALTER TABLE SQLExch_%s_DEL\r\n"
			"DROP COLUMN [%s]", sTable, sColumn);
		lpCSQL->ExecuteNonQuery(sSQL);

		sprintf(sSQL, "ALTER TABLE SQLExch_%s_DEL\r\n"
			"ADD [%s] %s", sTable, sColumn, sType);
		lpCSQL->ExecuteNonQuery(sSQL);

		if(iRows == 1)
		{
			//Drop the temp column.
			sprintf(sSQL, "ALTER TABLE SQLExch_%s_DEL\r\n"
				"DROP COLUMN [%s_TEMP]", sTable, sColumn);
			lpCSQL->ExecuteNonQuery(sSQL);
		}
	}
	rsID.Close();

	//-------------------------------------------------------------
	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//This function is not complete.
bool DropTableIfExist(CSQL *lpCSQL, char *sTable)
{
    char sSQL[1024];
    
    sprintf(sSQL, "if exists (select * from dbo.sysobjects where id ="
		" object_id(N'[dbo].[%s]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)"
		" drop table [dbo].[%s]", sTable, sTable);

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CreateTriggers(CSQL *lpCSQL, char *sDB, char *sDBO, char *sTable, char **sPKs, int iPKs, char *sPKColList)
{
	char *sBaseSQL = NULL;

	char *sSQL = NULL;
	char *sColList = NULL;
	char sColName[255];

	int iColNameSz = 0;
	int iLen = 0;
	int iColType = 0;

	CRecordSet rsColumns;

	sSQL = (char *) calloc(sizeof(char), 1048576);
	sColList = (char *) calloc(sizeof(char), 1048576);

	//Get a list of columns in the table. Put the list into sColList[].
	//----------------------------------------------------------------------------------------
	sprintf(sSQL, "SELECT Name, xType FROM SysColumns WHERE ID ="
		" (SELECT ID FROM SysObjects WHERE Name = '%s')"
		" ORDER BY ColOrder", sTable);
	lpCSQL->Execute(sSQL, &rsColumns);

	if(rsColumns.RowCount > 0)
	{
		strcpy(sColList, "");

		while(rsColumns.Fetch())
		{
			rsColumns.sColumnEx(1, sColName, sizeof(sColName), &iColNameSz);
			iColType = rsColumns.lColumn(2);

			if(iColType == COL_TYPE_TEXT || iColType == COL_TYPE_NTEXT || iColType == COL_TYPE_IMAGE)
			{
				//strcat(sColList, "CONVERT(VARCHAR,");
				//strcat(sColList, sColName);
				//strcat(sColList, "),");
			}
			else {
				strcat(sColList, "[");
				strcat(sColList, sColName);
				strcat(sColList, "],");
			}
		}

		if((iLen = strlen(sColList)))
		{
			if(sColList[iLen - 1] == ',')
			{
				sColList[iLen - 1] = '\0';
			}
		}
	}
	
	rsColumns.Close();

	//Create the triggers.
	//----------------------------------------------------------------------------------------

	sBaseSQL = "CREATE TRIGGER [SQLExch_%s_INS_Trigger] ON [%s].[%s]\r\n"
		"FOR INSERT\r\n"
		"AS\r\n\r\n"
		"INSERT INTO [%s].[%s].[SQLExch_%s_INS] (%s)\r\nSELECT %s FROM Inserted\r\n";

	sprintf(sSQL, sBaseSQL, sTable, sDBO, sTable, sDB, sDBO, sTable, sColList, sColList);
	lpCSQL->ExecuteNonQuery(sSQL);

	//----------------------------
	
	sBaseSQL = "CREATE TRIGGER [SQLExch_%s_UPD_Trigger] ON [%s].[%s]\r\n"
		"FOR UPDATE\r\n"
		"AS\r\n\r\n"
		"INSERT INTO [%s].[%s].[SQLExch_%s_UPD] (%s)\r\nSELECT %s FROM Inserted\r\n";

	sprintf(sSQL, sBaseSQL, sTable, sDBO, sTable, sDB, sDBO, sTable, sColList, sColList);
	lpCSQL->ExecuteNonQuery(sSQL);

	//----------------------------

	sBaseSQL = "CREATE TRIGGER [SQLExch_%s_DEL_Trigger] ON [%s].[%s]\r\n"
		"FOR DELETE\r\n"
		"AS\r\n\r\n"
		"INSERT INTO [%s].[%s].[SQLExch_%s_DEL] (%s)\r\nSELECT %s FROM Deleted\r\n";

	sprintf(sSQL, sBaseSQL, sTable, sDBO, sTable, sDB, sDBO, sTable, sPKColList, sPKColList);
	lpCSQL->ExecuteNonQuery(sSQL);

	free(sSQL);
	free(sColList);

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CreateTriggerTable(CSQL *lpCSQL, char *sDB, char *sDBO, char *sTable)
{
	char **sPKs = NULL;
	int iPKs = 0;
	int iPK = 0;
	int iLen = 0;
	char *sPKColList = NULL;

	sPKs = GetPrimaryKeys(lpCSQL, sTable, &iPKs);

	if(iPKs == 0)
	{
		char sText[1024];
		sprintf(sText, "The table [%s] does not have nay Primary Keys. Cannot create the replication.", sTable);
		MessageBox(Replication_hWnd, sText, gsTitleCaption, NULL);
		FreePrimaryKeys(sPKs, iPKs);
		return false;
	}
	
	sPKColList = (char *) calloc(sizeof(char), 1048576);

	strcpy(sPKColList, "");
	while(iPK < iPKs)
	{
		strcat(sPKColList, "[");
		strcat(sPKColList, sPKs[iPK]);
		strcat(sPKColList, "],");
		iPK++;
	}

	if((iLen = strlen(sPKColList)))
	{
		if(sPKColList[iLen - 1] == ',')
		{
			sPKColList[iLen - 1] = '\0';
		}
	}

	DropTriggers(lpCSQL, sTable);
	DropTables(lpCSQL, sDB, sDBO, sTable);
	CreateTables(lpCSQL, sDB, sDBO, sTable, sPKs, iPKs, sPKColList);
	CreateTriggers(lpCSQL, sDB, sDBO, sTable, sPKs, iPKs, sPKColList);

	FreePrimaryKeys(sPKs, iPKs);
	free(sPKColList);

	return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

char **GetPrimaryKeys(CSQL *pcSQL, char *sTableName, int *iNumOfKeys)
{
	int iPKCount = 0;
	int iPKColLen = 0;

	char sSQL[1024];
	char sPKCol[256];
	char *sBaseSQL = NULL;
	char **sOutKeys = NULL;

    CRecordSet rsTemp;

	sBaseSQL = "SELECT sysobjects.parent_obj AS tableid, 'PK' AS constraintType,"
		" sysobjects.name AS constraintName, syscolumns.name AS keyColumn, sysindexkeys.KeyNo"
		" FROM sysobjects"
		" INNER JOIN sysindexes ON sysobjects.name = sysindexes.name"
		" INNER JOIN sysindexkeys ON sysindexes.indid = sysindexkeys.indid AND sysindexes.id = sysindexkeys.id"
		" INNER JOIN syscolumns ON sysindexkeys.colid = syscolumns.colid AND sysobjects.parent_obj = syscolumns.id"
		" WHERE sysobjects.xtype = 'PK' and sysobjects.parent_obj = object_id('%s')"
		" Order by sysindexkeys.KeyNo";

	sprintf(sSQL, sBaseSQL, sTableName);

	if(!pcSQL->Execute(sSQL, &rsTemp))
	{
		return NULL;
	}

	if(rsTemp.RowCount > 0)
	{
		sOutKeys = (char **) calloc(sizeof(char *), rsTemp.RowCount + 1);

		while(rsTemp.Fetch())
		{
			rsTemp.sColumnEx(4, sPKCol, sizeof(sPKCol), &iPKColLen);

			if(iPKColLen > 0)
			{
				sOutKeys[iPKCount] = (char *) calloc(sizeof(char), iPKColLen + 1);
				strcpy(sOutKeys[iPKCount], sPKCol);
				iPKCount++;
			}
		}
		sOutKeys[iPKCount] = NULL;
	}
	else{
		sOutKeys = NULL;
	}

	rsTemp.Close();
	
	*iNumOfKeys = iPKCount;

	return sOutKeys;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FreePrimaryKeys(char **sOutKeys, int iKeys)
{
	int iPos = 0;

	while(iPos < iKeys)
	{
		free(sOutKeys[iPos]);
		iPos++;
	}

	if(iKeys > 0)
	{
		free(sOutKeys);
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool GenSQLScriptsForOperation(CSQL *lpCSQL)
{
	char **sPKs = NULL;
	int iPKs = 0;

	char sColList[10240];
	char sSQL[10240];
	char sTemp[10240];
	char sTable[255];
	int iTempSz = 0;
	int iColType = 0;
	int iLen = 0;
	long lSeq = 1000;

	FILE *hSource = NULL;

	CRecordSet rsTables;
	CRecordSet rsColumns;

	sprintf(sTemp, "%s\\SQL\\SQLExch_Statements.sql", gsPath);
	if((hSource = fopen(sTemp, "rb")) == NULL)
	{
		MessageBox(MainDialog_hWnd, "Failed to open the SQLExch_Statements file.", gsTitleCaption, 0);
		return false;
	}

	iLen = Get_FileSize(sTemp);
	fread(sTemp, iLen, sizeof(char), hSource);
	sTemp[iLen] = '\0';

	fclose(hSource);

	lpCSQL->ExecuteNonQuery("DROP TABLE SQLExch_Statements"); //Recreate the SQLExch_Statements table.
	lpCSQL->ExecuteNonQuery(sTemp); //Recreate the SQLExch_Statements table.

	strcpy(sSQL, "SELECT Name"
		" FROM SysObjects"
		" WHERE xType = 'U' AND (LEFT(NAME, 8) <> 'SQLExch_'"
		" AND ID IN (SELECT Parent_Obj FROM SysObjects AS A WHERE A.xType = 'TR' AND LEFT(A.NAME, 8) = 'SQLExch_'))"
		//" OR Name = 'SQLExch_Statements'"
		" ORDER BY Name");
	lpCSQL->Execute(sSQL, &rsTables);

	lpCSQL->ExecuteNonQuery("DELETE FROM SQLExch_Statements");

	while(rsTables.Fetch())
	{
		strcpy(sColList, "");

		rsTables.sColumnEx(1, sTable, sizeof(sTable), &iTempSz);

		sprintf(sSQL, "SELECT Name, xType FROM SysColumns WHERE ID ="
			" (SELECT ID FROM SysObjects WHERE Name = '%s')"
			" ORDER BY ColOrder", sTable);
		lpCSQL->Execute(sSQL, &rsColumns);

		while(rsColumns.Fetch())
		{
			rsColumns.sColumnEx(1, sTemp, sizeof(sTemp), &iTempSz);
			iColType = rsColumns.lColumn(2);

			if(iColType == COL_TYPE_TEXT || iColType == COL_TYPE_NTEXT || iColType == COL_TYPE_IMAGE)
			{
				//strcat(sColList, "CONVERT(VARCHAR,");
				//strcat(sColList, sTemp);
				//strcat(sColList, "),");
			}
			else {
				strcat(sColList, "[");
				strcat(sColList, sTemp);
				strcat(sColList, "],");
			}
		}

		if((iLen = strlen(sColList)))
		{
			if(sColList[iLen - 1] == ',')
			{
				sColList[iLen - 1] = '\0';
			}
		}

		//--------------------------------------------------------------------------------------------------------
		sprintf(sSQL, "INSERT INTO SQLExch_Statements (Statement, OnSuccess, OnFailure, ImportTable,"
			" Sequence, Active, Comments, Identifier, Dirty)"
			" VALUES('UPDATE SQLExch_%s_INS SET SQLExch_Pending = 1', NULL, NULL, '', '%d', '1', 'Auto Generated.', 'SQLExch_Pending_Update_%d', 1)",
			sTable, lSeq, lSeq);
		lpCSQL->ExecuteNonQuery(sSQL);
		lSeq = (lSeq + 10);

		strcpy(sTemp, "SELECT ");
		strcat(sTemp, sColList);
		strcat(sTemp, " FROM ");
		strcat(sTemp, " SQLExch_");
		strcat(sTemp, sTable);
		strcat(sTemp, "_INS WHERE SQLExch_Pending = 1");

		sprintf(sSQL, "INSERT INTO SQLExch_Statements (Statement, OnSuccess, OnFailure, ImportTable,"
			" Sequence, Active, Comments, Identifier, Dirty)"
			" VALUES('%s', 'DELETE FROM SQLExch_%s_INS WHERE SQLExch_Pending = 1', NULL, '%s', '%d', '1', 'Auto Generated.', 'Insert', 1)",
			sTemp, sTable, sTable, lSeq);
		lpCSQL->ExecuteNonQuery(sSQL);
		lSeq = (lSeq + 10);

		//--------------------------------------------------------------------------------------------------------
		sprintf(sSQL, "INSERT INTO SQLExch_Statements (Statement, OnSuccess, OnFailure, ImportTable,"
			" Sequence, Active, Comments, Identifier, Dirty)"
			" VALUES('UPDATE SQLExch_%s_UPD SET SQLExch_Pending = 1', NULL, NULL, '', '%d', '1', 'Auto Generated.', 'SQLExch_Pending_Update_%d', 1)",
			sTable, lSeq, lSeq);
		lpCSQL->ExecuteNonQuery(sSQL);
		lSeq = (lSeq + 10);

		strcpy(sTemp, "SELECT ");
		strcat(sTemp, sColList);
		strcat(sTemp, " FROM ");
		strcat(sTemp, " SQLExch_");
		strcat(sTemp, sTable);
		strcat(sTemp, "_UPD WHERE SQLExch_Pending = 1");

		sprintf(sSQL, "INSERT INTO SQLExch_Statements (Statement, OnSuccess, OnFailure, ImportTable,"
			" Sequence, Active, Comments, Identifier, Dirty)"
			" VALUES('%s', 'DELETE FROM SQLExch_%s_UPD WHERE SQLExch_Pending = 1', NULL, '%s', '%d', '1', 'Auto Generated.', 'Update', 1)",
			sTemp, sTable, sTable, lSeq);
		lpCSQL->ExecuteNonQuery(sSQL);
		lSeq = (lSeq + 10);

		//--------------------------------------------------------------------------------------------------------
		sprintf(sSQL, "INSERT INTO SQLExch_Statements (Statement, OnSuccess, OnFailure, ImportTable,"
			" Sequence, Active, Comments, Identifier, Dirty)"
			" VALUES('UPDATE SQLExch_%s_DEL SET SQLExch_Pending = 1', NULL, NULL, '', '%d', '1', 'Auto Generated.', 'SQLExch_Pending_Update_%d', 1)",
			sTable, lSeq, lSeq);
		lpCSQL->ExecuteNonQuery(sSQL);
		lSeq = (lSeq + 10);

		//Get a list of primary keys for the import table.
		sPKs = GetPrimaryKeys(lpCSQL, sTable, &iPKs);
		if(iPKs > 0)
		{
			int iPK = 0;

			//---------------------------------
			//Make a list of the primary keys.
			//---------------------------------
			strcpy(sColList, "");
			while(iPK < iPKs)
			{
				strcat(sColList, "[");
				strcat(sColList, sPKs[iPK]);
				strcat(sColList, "],");
				iPK++;
			}

			if((iLen = strlen(sColList)))
			{
				if(sColList[iLen - 1] == ',')
				{
					sColList[iLen - 1] = '\0';
				}
			}
			//---------------------------------

			strcpy(sTemp, "SELECT ");
			strcat(sTemp, sColList);
			strcat(sTemp, " FROM ");
			strcat(sTemp, " SQLExch_");
			strcat(sTemp, sTable);
			strcat(sTemp, "_DEL WHERE SQLExch_Pending = 1");

			sprintf(sSQL, "INSERT INTO SQLExch_Statements (Statement, OnSuccess, OnFailure, ImportTable,"
				" Sequence, Active, Comments, Identifier, Dirty)"
				" VALUES('%s', 'DELETE FROM SQLExch_%s_DEL WHERE SQLExch_Pending = 1', NULL, '%s_SQLExch_Delete', '%d', '1', 'Auto Generated.', 'Delete', 1)",
				sTemp, sTable, sTable, lSeq);
			lpCSQL->ExecuteNonQuery(sSQL);
			lSeq = (lSeq + 10);
		}

		FreePrimaryKeys(sPKs, iPKs);

		rsColumns.Close();
	}		

	rsTables.Close();

	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
